
;##########################################################################################
;###########      S/C Potential Calibration to Electron Plasma Densities       ############
;##########################################################################################

; This code is intended to calibrate spacecraft potential measurements to electron plasma densities
; using data from the Magnetopsheric MultiScale Mission (MMS)

; In order for the code to work, the SPEDAS environment has to be installed, details can be found here:
; http://spedas.org/wiki/index.php?title=Main_Page

; Before running the code for the first time, use "mms_init" to initialize MMS SPEADS environment:
mms_init

; Also make sure following routines are compiled in your IDL:
;  spin_rem.pro, expfit1.pro, expfit2.pro, exp_fit_1, exp_fit_2 as well as the MPFIT routines

; If you have questions you can contact Daniel Teubenbacher: daniel.teubenbacher@oeaw.ac.at

;##########################################################################################
; Following parameters shall be defined/altered by the user:

; Choose local data directory for MMS data:
local_data_dir="M:\spedas\mms\"

; Choose directory where to save the derived density data:
data_dir="C:\Users\dteubenbacher\IDLWorkspace\"

; Choose MMS probe (possibilities: '1', '2', '3', '4'):
probe = '1'

; Set time range ('YYYY-MM-DD/HH:MM:SS'):
; some examples:
;tr = ['2019-04-23/18:00:00', '2019-04-23/20:30:00']
;tr = ['2021-06-25/00:00:00', '2021-06-25/05:00:00']
;tr = ['2021-02-02/08:00:00', '2021-02-02/10:00:00']
;tr = ['2019-02-05/06:00:00', '2019-02-05/08:00:00']
tr = ['2019-02-26/14:00:00', '2019-02-26/17:00:00']


; Choose model function for average photolectron curve derivation:
;   Gaussian:               0   (not reccomended, just for troubleshooting/first guess)
;   one exponential term:   1
;   two exponential term:   2   (default)

model_function = 2


; Show average photoelectron curve plot (yes=1, no=0):
show_plot = 1

; Perform spin tone removal of spacecraft potential data (suggested):
; (requires to log in to MMS SDC as the EDP phase data is not publically available)
spin_rem = 1      ; (yes=1, no=0)






;##########################################################################################
;##########################################################################################

; Load MMS data: 

tic ;start timer
mms_load_aspoc, trange=tr, local_data_dir=local_data_dir, probes=probe, datatype='aspoc', level='l2', data_rate='srvy', /time_clip
mms_load_edp, trange=tr, local_data_dir=local_data_dir, probes=probe, datatype=['scpot','dce'], level=['l2'], data_rate='fast', /time_clip       
mms_load_fpi, trange=tr, local_data_dir=local_data_dir, probes=probe, datatype=['des-moms','dis-moms'], level='l2', data_rate='fast', /time_clip

; load fluxgate magentometer (FGM) data:
;mms_load_fgm, trange=tr, local_data_dir="M:\spedas\mms\", probes=probe, /time_clip



; save MMS data:

; ASPOC parameters:
get_data, 'mms'+probe+'_aspoc_ionc_l2', data=ionc, dlimits=dl, limits=l
store_data, 'ionc', data={x: ionc.X, y: ionc.Y}, dlimits=dl, limits=l

get_data, 'mms'+probe+'_asp1_ionc_l2', data=ionc1, dlimits=dl, limits=l
store_data, 'mms'+probe+'_asp1_ionc_l2', data={x: ionc1.X, y: ionc1.Y}
get_data, 'mms'+probe+'_asp2_ionc_l2', data=ionc2, dlimits=dl, limits=l
store_data, 'mms'+probe+'_asp2_ionc_l2', data={x: ionc2.X, y: ionc2.Y}

get_data, 'mms'+probe+'_asp1_energy_l2', data=e1
store_data, 'mms'+probe+'_asp1_energy_l2', data={x: e1.X, y: e1.Y}
get_data, 'mms'+probe+'_asp2_energy_l2', data=e2
store_data, 'mms'+probe+'_asp2_energy_l2', data={x: e2.X, y: e2.Y}

store_data, 'mms'+probe+'_aspoc_ionc_l2', data={x: ionc1.X, y: [[ionc1.Y],[ionc2.Y]], labels:['ASPOC 1','ASPOC 2']}
store_data, 'mms'+probe+'_aspoc_energy_l2', data={x: e1.X, y: [[e1.Y],[e2.Y]], labels:['ASPOC 1','ASPOC 2']}

; plasma parameters from FPI:
fpi_datarate='fast'
get_data, 'mms'+probe+'_des_numberdensity_'+fpi_datarate, data=density, dlimits=dl, limits=l
store_data, 'e_density', data={x: density.X, y: density.Y}, dlimits=dl, limits=l
get_data, 'mms'+probe+'_dis_numberdensity_'+fpi_datarate, data=idensity, dlimits=dl11, limits=l11
store_data, 'i_density', data={x: idensity.X, y: idensity.Y}, dlimits=dl11, limits=l11

get_data, 'mms'+probe+'_des_temppara_'+fpi_datarate, data=temppara, dlimits=dl, limits=l
store_data, 'e_temppara', data={x: temppara.X, y: temppara.Y}, dlimits=dl, limits=l
get_data, 'mms'+probe+'_des_tempperp_'+fpi_datarate, data=tempperp, dlimits=dl, limits=l
store_data, 'e_tempperp', data={x: tempperp.X, y: tempperp.Y}, dlimits=dl, limits=l

temp_all = (temppara.Y + 2*tempperp.Y) / 3
store_data, 'e_temp', data={x: tempperp.X, y: temp_all}, dlimits=dl, limits=l


; S/C potential from EDP:
get_data, 'mms'+probe+'_edp_scpot_fast_l2', data=scpot, dlimits=dl, limits=l
store_data, 'sc_pot', data={x: scpot.X, y: scpot.Y}, dlimits=dl, limits=l





; spline interpolation of data to create an evenly spaced timeline for e density, e temp and sc pot:


dur = (time_double(tr(1))-time_double(tr(0))) / 60               ; duration of time interval in minutes

; time vector for dur:
; time shifted by 1. time value of original data ; #### 32 Hz steps #####
k_32 = dur*60D*32D
t_new_32 = DINDGEN(k_32)/32D + density.X[1]     

; time shifted by 1. time value of original data ; #### steps every 4.5 sec #####
t_new = DINDGEN(dur*60/4.5)*4.5 + density.X[1]     


density_new = Interpol(density.Y, density.X, t_new, /spline)
store_data, 'e_density_new', data={x: t_new, y: density_new}, dlimits=dl, limits=l

temp_new = Interpol(temp_all, tempperp.X, t_new, /spline)
temp_new_32 = Interpol(temp_all, tempperp.X, t_new_32, /spline)
store_data, 'e_temp_new', data={x: t_new, y: temp_new}, dlimits=dl, limits=l


if spin_rem eq 0 then begin
  scpot_new = Interpol(scpot.Y, scpot.X, t_new, /spline)
  scpot_new_32 = Interpol(scpot.Y, scpot.X, t_new_32, /spline)

endif else begin  
  scpot_data = scpot.Y    ;perform spin removal
  scpot_time = scpot.X
  scpot_nospin = spin_rem(tr, probe, scpot_time, scpot_data)   
  scpot_new = Interpol(scpot_nospin, scpot.X, t_new, /spline)
  scpot_new_32 = Interpol(scpot_nospin, scpot.X, t_new_32, /spline)
  
endelse


store_data, 'sc_pot_new', data={x: t_new, y: scpot_new}, dlimits=dl, limits=l

ionc_new = Interpol(ionc.Y, ionc.X, t_new, /spline)
ionc_new_32 = Interpol(ionc.Y, ionc.X, t_new_32, /spline)
store_data, 'ionc_new', data={x: t_new, y: ionc_new}, dlimits=dl, limits=l


; defining some constants:
A_spac = 34            ; m^2
m_e = 9.10938356e-31   ; kg
k_B = 1.38064852e-23   ; m^2kgs^-2K-1
q = -1.60217662e-19     ; C
pi = 3.14159265359


;convert units:
temp_new_si = temp_new * ABS(q)/ k_B    ; in Kelvin
temp_new_si_32 = temp_new_32 * ABS(q)/ k_B    ; in Kelvin
density_new_si = density_new*1e6

;compute electron current:
I_e_si = - ((A_spac*q)/(SQRT(2*m_e*pi))) * density_new_si * SQRT(k_B*temp_new_si)*(1+((ABS(q)*scpot_new)/(k_B*temp_new_si)))
; convert A to µA:
I_e = I_e_si * 1e6

store_data, 'I_e', data={x: t_new, y: I_e}, dlimits=dl, limits=l

;compute photoelectron current:
I_phot = (I_e + ionc_new)
store_data, 'I_phot', data={x: t_new, y: I_phot}, dlimits=dl, limits=l


; compute average photoelectron curve:

if model_function eq 0 then begin
  ; first guess: gaussian non-linear least squares fit:
  I_phot_32 = Interpol(I_phot, t_new, t_new_32, /spline)
  gauss = gaussfit(scpot_new, I_phot, nterms=6)
  gauss_32 = gaussfit(scpot_new_32, I_phot_32, nterms=6)
  avg_phot = gauss
  avg_phot_32 = gauss_32
  tit = 'gaussian'
  fitused='################   Gaussian Fit used!   ################'
endif

if model_function eq 1 then begin   ; EDIT!!!
  ; usage of 1 exponential function with 2 coefficients:
  ; define starting values for fitting routine (mpfitfun):
    start_values = [200.D, 40.D]            ;(default, if derivation of average photoelectron curve incorrect, please adjust)
    result = exp_fit_1(tr, probe, scpot_new, I_phot, show_plot, start_values)
    avg_phot = expfit1(scpot_new, result)
    avg_phot_32 = expfit1(scpot_new_32, result)
    tit = '1 exp.'
    fitused='###################   1 Exponential Fit used!   ######################'
endif

if model_function eq 2 then begin
  ; usage of 2 exponential functions with 4 coefficients:
  ; define starting values for fitting routine (mpfitfun):
  start_values = [50D, 1D, 500D, 0.2D]            ;(default, if derivation of average photoelectron curve incorrect, please adjust)
  result = exp_fit_2(tr, probe, scpot_new, I_phot, show_plot, start_values)
  avg_phot = expfit2(scpot_new, result)
  avg_phot_32 = expfit2(scpot_new_32, result)
  tit = '2 exp.'
  fitused='###################   2 Exponential Fit used!   ######################'
endif



; compute electron density at FPI time resolution (4.5sec) for comparison and also in 32 Hz:

N_e_si = (SQRT(2*m_e*pi)/(A_spac*ABS(q)))*(1/SQRT(k_B*temp_new_si))*(1/(1+(ABS(q)*scpot_new)/(k_B*temp_new_si)))*((avg_phot*1e-6) - (ionc_new*1e-6))                     ; 4.5 sec
N_e_si_32 = (SQRT(2*m_e*pi)/(A_spac*ABS(q)))*(1/SQRT(k_B*temp_new_si_32))*(1/(1+(ABS(q)*scpot_new_32)/(k_B*temp_new_si_32)))*((avg_phot_32*1e-6) - (ionc_new_32*1e-6))   ; 32 Hz

; convert m^-3 to cm^-3:
N_e = N_e_si / 1e6
N_e_32 = N_e_si_32 / 1e6

store_data, 'N_e', data={x: t_new, y: N_e}, dlimits=dl, limits=l
store_data, 'N_e_32', data={x: t_new, y: N_e_32}, dlimits=dl, limits=l

;density_new = density_new - 2.7D   ; average distance from electron and ion density from mms1

store_data, 'electron_density', data={x: t_new, y: [[N_e],[density_new]], labels:['n!De!N derived','n!De!N measured']}
options, 'electron_density', ysubtitle=['[cm^-3]']


; plot for difference of measured and calculated electron density:
; first compute difference (absolut value):
; normalized:

diff = 100*(N_e - density_new) / density_new    ; Ne ord density_new

;store_data, 'electron_density_differnce_(cm^-3)', data={x: t_new, y: diff, labels:['N_e derived - N_e measured']}, dlimits=dl, limits=l
store_data, 'electron_density_difference', data={x: t_new, y: diff}
options, 'electron_density_difference', ysubtitle=['[%]']




; make a plot:

window, 1
tplot, window=1, title='MMS'+probe+': Multipanel Plot from '+tr[0]+' to '+tr[1], $
  'electron_density_difference'
ylim, 'electron_density_difference', -100,100,0
tplot, 'electron_density', /add
;ylim, 'electron_density', 0,12,0

; e Temp:
options, 'e_temp', 'ytitle', 'electron_temp'
options, 'e_temp', labels=['(T!Dpara!N + 2T!Dperp!N)/3']
tplot, 'e_temp', /add
;  ylim, 'e_temp', 10,5000,1   ; log scale for whole time range
;ylim, 'e_temp', 25,40,0

tplot, 'sc_pot', /add
ylim, 'electron_density', 0,70,0

tplot, 'mms'+probe+'_aspoc_ionc_l2', /add
ylim, 'mms'+probe+'_aspoc_ionc_l2', 8,12, 0
;tplot, 'mms'+probe+'_aspoc_energy_l2', /add
;ylim, 'mms'+probe+'_aspoc_energy_l2', 4,8,0

;tplot, 'mms'+probe+'_des_energyspectr_omni_fast', /add
tplot, 'mms'+probe+'_edp_dce_gse_fast_l2', /add
ylim, 'mms'+probe+'_edp_dce_gse_fast_l2', -10,10,0




timebar, 0.0, /databar, varname='electron_density_difference', linestyle=0, color=!d.n_colors-3  ; horizontal line
timebar, 0.0, /databar, varname='mms'+probe+'_edp_dce_gse_fast_l2', linestyle=0  ; horizontal line
timebar, 0.0, /databar, varname='mms'+probe+'_fgm_b_gse_srvy_l2_bvec', linestyle=0  ; horizontal line
;timebar, '2019-02-26/15:44:00'   ; vertical line
;timebar, '2019-02-26/14:32:00'   ; vertical line
;timebar, '2019-02-26/14:50:00'   ; vertical line

tplot_options, 'xmargin', [20, 20]
options, '*', 'labflag', -1     ;label spacing
time_stamp, /off



; another plot for comparison:
; choose a time interval that is 10% of the total duration of the defined interval:
comp_plot_1 = plot(t_new[0:n_elements(N_e)/20]-time_double(tr(0)), density_new[0:n_elements(density_new)/20], thick=2, $
  title='Comparison of FPI measurement (black, 4.5 sec) and calibrated s/c potential !C(green, 32 Hz) in the first 5% of the total time intervall', $
  xtitle='time in sec since beginning of time range', ytitle='electron density [cm!U-3!N]', xrange=[0,((max(t_new_32)-time_double(tr(0)))/20)])
comp_plot_2 = plot(t_new_32[0:n_elements(N_e_32)/20]-time_double(tr(0)), N_e_32[0:n_elements(N_e_32)/20], thick=1.5, color='g', /overplot)


print, '######################################################################'
print, fitused
if isa(result) eq 1 then print, '###   fitting param.: ', result, '###'
if spin_rem eq 1 then print, '###################   Spin Tone Removal applied   ##################' else print, '################   no Spin Tone Removal applied   ################'

avg_diff = total(diff[0:(size(diff, /n_elements)-2)])/(size(diff, /n_elements)-1)
print, "Average difference between meas. and calc. electron density: ", avg_diff, '%'
if avg_diff gt 10 then print, "HIGH AVERAGE DIFF. (above 10%): Check photoelectron curve and spin tone removal plots and possibly change start values for fitting routine. Narrowing the time interval might also lead to improvements."

print, "density data is being saved to: " + data_dir + " . . ." 
; save and export the density data:
p0 = string(tr(0))
p0 = p0.Replace('-','')
p0 = p0.Replace('/','_')
p0 = p0.Replace(':','')
p1 = string(tr(1))
p1 = p1.Replace('-','')
p1 = p1.Replace('/','_')
p1 = p1.Replace(':','')

time = time_string(t_new_32,tformat="YYYY-MM-DD/hh:mm:ss.fff")
fname= data_dir+'MMS'+probe+'_ne_fast_'+p0+'_'+p1+'.txt'

N_e_32[where(N_e_32 gt 300)] = 0
N_e_32[where(N_e_32 lt 0)] = 0

A = transpose([[string(time)], [string(N_e_32)]])

openw,1,fname
printf,1,FORMAT='("Time,",4X,"ne(cm^-3)")'
printf,1,  A , FORMAT='(A23, 4x, A16)'
close, 1

; save as IDL .sav file:
save, t_new_32, N_e_32, filename= data_dir+'MMS'+probe+'_ne_fast_'+p0+'_'+p1+'.sav', description="t_new_32: time vector in [sec], N_e_32: calibrated electron density in [cm^-3]"

print, '######################################################################'
print, "Electron density data saved as .txt and .sav file!"

toc ;end timer









end
