

; Function to perform spin tone removal intended for spacecraft potential measurements
; dervies an average spin model for the chosen time period and fits 10 superposed sine curves to compute an average spin model
; works only in combination with 'density_derivation_ASPOC_ON.pro'

; input variables:
;       tr: time range
;       probe: either 1, 2, 3 or 4
;       scpot_time: time vector from s/c potential measurements
;       scpot_data: s/c potential data vector matching the time vector
;
; return:
;       scpot_nospin: scpot_data with the spin tone removed
;       
; other output:
;        plot demonstrating the steps of the spin model derivation 

;##########################################################################################
;##########################################################################################

; Load spin phase data from EDP:
; (this does not work if user: public !)

function spin_rem, tr, probe, scpot_time, scpot_data

mms_load_edp, trange=tr, local_data_dir=local_data_dir, probes=probe, level=['l2pre'], data_rate='fast', /time_clip, always_prompt=1

get_data, 'mms'+probe+'_edp_phase_fast_l2pre', data=edp_phase, dlimits=dl, limits=l
store_data, 'edp_phase', data={x: edp_phase.X, y: edp_phase.Y}, dlimits=dl, limits=l



data = scpot_data  ;scpot.Y
time = scpot_time   ;scpot.X ;t_vec
phase_new = edp_phase.Y



; average s/c pot (can be changed to moving average:
; used to get spacecraft fluctuations:
scpot_avg = total(data)/n_elements(data)

; s/c pot fluctuatiuons:
;scpot_delta = scpot.Y - scpot_avg   ; take one average value
scpot_delta = data - SMOOTH(data, 6000)          ; use a moving average to remove trends      


 
; phase plot without binning:
phase_plot = scatterplot(phase_new, scpot_delta, $ ;or phase_new
    symbol='X', sym_size=0.6, $
    title='MMS'+probe+' Potential Fluctuations'+tr(0)+' to '+tr(1), $
    XTITLE='s/c spin phase angle [°]', YTITLE='$\delta V_{sc}$', $
    xrange=[0,360], xtickinterval=90, sym_color='blue', yrange=[-0.3,0.3],font_size=15)


; binning angle values to half a degree bins:

phase_bins=0.5D*findgen(720)
scpot_delta_binned=dblarr(720)

bincenter=dblarr(719)
Error1=dblarr(720)
current1 = scpot_delta
for i=0,718 do begin
    ;print, i
    elementsInthisBin=where(phase_new gt phase_bins(i) and phase_new lt phase_bins(i+1))
    ;print, elementsInthisBin
    bincenter(i)=(phase_bins(i) + phase_bins(i+1))/2.
    if n_elements(elementsInthisBin) gt 1 then begin
      scpot_delta_binned(i)=mean(current1(elementsInthisBin))
      Error1(i)=stddev(current1(elementsInthisBin) - scpot_delta_binned(i))   ;überdenken
    endif
endfor

; phase plot with binning:
phase_plot_binned = errorplot(phase_bins, scpot_delta_binned, Error1, $
    symbol='X', sym_size=0.1, errorbar_color="black", errorbar_thick=0.01, $
    errorbar_capsize=0.05,/overplot)

;overplot over errorbars:
overplot = scatterplot(phase_bins, scpot_delta_binned, symbol="star", /sym_filled, sym_color="green", /overplot)



; #############################################################################################
; ###########################       Fit the binned Spin Data      #############################
; #############################################################################################
; using 10 sine functions, 3 parameters per function:
; if spin model fit is bad: try using other starting values:

start = [0.05, 100, 1, 0.06, 200, 1, 0.05, 50, 1, 0.04, 30, 1,   0.05, 100, 1, 0.06, 200, 1, 0.05, 50, 1, 0.04, 30, 1, 0.02, 100, 1, 0.06, 200, 1]
  ;start = [0.5, 100, 1, 0.06, 200, 1, 0.05, 500, 1, 0.4, 300, 1,   0.05, 100, 1, 0.06, 200, 1, 0.05, 50, 1, 0.04, 30, 1, 0.02, 100, 1, 0.06, 200, 1]
  ;start = [0.05, 10, 1, 0.06, 200, 1, 0.05, 50, 1, 0.4, 30, 0.1,   0.05, 100, 1, 0.06, 20, 1, 0.5, 50, 1, 0.4, 300, 1, 0.02, 10, 1, 0.06, 2, 1]            
  ;start = [0.5, 1000, 1, 0.6, 20, 10, 0.05, 500, 1, 0.4, 30, 1,   0.05, 100, 1, 0.06, 200, 1, 0.05, 50, 1, 0.4, 30, 1, 0.02, 10, 1, 0.06, 20, 1]


; boundaries:
pi = replicate({fixed:0, limited:[0,0], limits:[0.D,0.D]},30)
pi(0).limited(0) = 1     ; lower boundary activated
pi(0).limits(0) = 0.05
pi(3).limited(0) = 1     ; lower boundary activated
pi(3).limits(0) = 0.01
pi(6).limited(0) = 1     ; lower boundary activated
pi(6).limits(0) = 0.005
;pi(6).limited(0) = 1     ; lower boundary activated
;pi(6).limits(0) = 0.01
;pi(8).limited(0) = 1     ; lower boundary activated
;pi(8).limits(0) = 0.01

expr = 'P[0] * SIN(2*!DPI*X/P[1] + 2*!DPI/P[2]) + P[3] * SIN(2*!DPI*X/P[4] + 2*!DPI/P[5]) + P[6] * SIN(2*!DPI*X/P[7] + 2*!DPI/P[8])+ P[9] * SIN(2*!DPI*X/P[10] + 2*!DPI/P[11]) + P[12] * SIN(2*!DPI*X/P[13] + 2*!DPI/P[14]) + P[15] * SIN(2*!DPI*X/P[16] + 2*!DPI/P[17]) + P[18] * SIN(2*!DPI*X/P[19] + 2*!DPI/P[20]) + P[21] * SIN(2*!DPI*X/P[22] + 2*!DPI/P[23]) + P[24] * SIN(2*!DPI*X/P[25] + 2*!DPI/P[26])+ P[27] * SIN(2*!DPI*X/P[28] + 2*!DPI/P[29])'

fit_coeff = MPFITEXPR(expr, phase_bins, scpot_delta_binned, stddev(scpot_delta_binned), start, PARINFO=pi, quiet=quiet)

;evaluate:
X = phase_bins
fit_data = fit_coeff[0] * SIN(2*!DPI*X/fit_coeff[1] + 2*!DPI/fit_coeff[2]) + fit_coeff[3] * SIN(2*!DPI*X/fit_coeff[4] + 2*!DPI/fit_coeff[5]) + $
    fit_coeff[6] * SIN(2*!DPI*X/fit_coeff[7] + 2*!DPI/fit_coeff[8]) + fit_coeff[9] * SIN(2*!DPI*X/fit_coeff[10] + 2*!DPI/fit_coeff[11]) + $
    fit_coeff[12] * SIN(2*!DPI*X/fit_coeff[13] + 2*!DPI/fit_coeff[14]) + fit_coeff[15] * SIN(2*!DPI*X/fit_coeff[16] + 2*!DPI/fit_coeff[17]) + $
    fit_coeff[18] * SIN(2*!DPI*X/fit_coeff[19] + 2*!DPI/fit_coeff[20]) + fit_coeff[21] * SIN(2*!DPI*X/fit_coeff[22] + 2*!DPI/fit_coeff[23]) + $
    fit_coeff[24] * SIN(2*!DPI*X/fit_coeff[25] + 2*!DPI/fit_coeff[26]) + fit_coeff[27] * SIN(2*!DPI*X/fit_coeff[28] + 2*!DPI/fit_coeff[29])


; phase fitting plot:
phase_fit = PLOT(phase_bins, fit_data, /overplot, color="red", thick=2)

; #############################################################################################
; ######################          REMOVE SPIN FROM S/C POT           ##########################
; #############################################################################################

;make fit data function with input
X = phase_new
fit_data_all = fit_coeff[0] * SIN(2*!DPI*X/fit_coeff[1] + 2*!DPI/fit_coeff[2]) + fit_coeff[3] * SIN(2*!DPI*X/fit_coeff[4] + 2*!DPI/fit_coeff[5]) + $
    fit_coeff[6] * SIN(2*!DPI*X/fit_coeff[7] + 2*!DPI/fit_coeff[8]) + fit_coeff[9] * SIN(2*!DPI*X/fit_coeff[10] + 2*!DPI/fit_coeff[11]) + $
    fit_coeff[12] * SIN(2*!DPI*X/fit_coeff[13] + 2*!DPI/fit_coeff[14]) + fit_coeff[15] * SIN(2*!DPI*X/fit_coeff[16] + 2*!DPI/fit_coeff[17]) + $
    fit_coeff[18] * SIN(2*!DPI*X/fit_coeff[19] + 2*!DPI/fit_coeff[20]) + fit_coeff[21] * SIN(2*!DPI*X/fit_coeff[22] + 2*!DPI/fit_coeff[23]) + $
    fit_coeff[24] * SIN(2*!DPI*X/fit_coeff[25] + 2*!DPI/fit_coeff[26]) + fit_coeff[27] * SIN(2*!DPI*X/fit_coeff[28] + 2*!DPI/fit_coeff[29])




;substract spin related fluctuations from measured spacecraft potential vom EDP:
scpot_nospin = data - fit_data_all    ; new scpot data in 32 Hz

;save, scpot_nospin, filename=v_sav, description="spacecraft potential with spin tone removal"

return, scpot_nospin



end